use clap::Parser;
use colored::Colorize;
use crate::commands::Cli;
use log::{SetLoggerError, trace};
use std::env;

mod commands;
mod package;
mod tmp;

fn main() {
    setup_logger().expect("Unable to setup logger.");

    #[cfg(not(debug_assertions))]
    {
        #[cfg(target_family = "unix")]
        if unsafe { libc::getuid() } != 0 {
            use log::error;
            error!("pkgr must be run as root.");
            std::process::exit(1);
        }
    }

    trace!("Parsing command line arguments...");
    let c = Cli::parse();
    trace!("Command line arguments: {:?}", c);

    trace!("Executing command...");
    c.command.execute();
    trace!("Command executed.");

    trace!("Exiting...");
}

fn setup_logger() -> Result<(), SetLoggerError> {
    fern::Dispatch::new()
        .format(|out, message, record| {
            out.finish(format_args!(
                "{} {}",
                // Some logic so messages look nice
                if message.to_string().len() > 0 {
                    match record
                        .level()
                        .to_string()
                        .chars()
                        .nth(0)
                        .unwrap_or('T')
                    {
                        'T' => "[TRACE]".cyan(),
                        'D' => "??".green(),
                        'I' => "=>".blue(),
                        'W' => "##".yellow(),
                        'E' => "!!".red(),
                        _ => "**".blue(),
                    }.to_string()
                } else { "".to_string() },
                message.to_string().bright_white()
            ))
        })
        .level(env::var("PKGR_LOG_LEVEL")
            .unwrap_or_else(|_| "info".to_string())
            .parse()
            .unwrap_or(log::LevelFilter::Info))
        .chain(std::io::stdout())
        .apply()
}
